@@ -24,7 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <locale.h>
 #include <stdlib.h>
@@ -32,7 +32,6 @@
 
 #include "gdkx.h"
 #include "gdk.h"		/* For gdk_flush() */
-#include "gdkx.h"
 #include "gdkpixmap.h"
 #include "gdkinternals.h"
 #include "gdkdisplay-x11.h"
