@@ -852,7 +852,7 @@ gtk_printer_set_is_default (GtkPrinter *printer,
 {
   g_return_if_fail (GTK_IS_PRINTER (printer));
 
-  printer->priv->is_default = TRUE;
+  printer->priv->is_default = val;
 }
 
 /**
@@ -1097,6 +1097,21 @@ list_added_cb (GtkPrintBackend *backend,
   return FALSE;
 }
 
+static void
+backend_status_changed (GObject    *object,
+                        GParamSpec *pspec,
+                        gpointer    data)
+{
+  GtkPrintBackend *backend = GTK_PRINT_BACKEND (object);
+  PrinterList *printer_list = data;
+  GtkPrintBackendStatus status;
+
+  g_object_get (backend, "status", &status, NULL);
+ 
+  if (status == GTK_PRINT_BACKEND_STATUS_UNAVAILABLE)
+    list_done_cb (backend, printer_list);  
+}
+
 static void
 list_done_cb (GtkPrintBackend *backend, 
 	      PrinterList     *printer_list)
@@ -1105,6 +1120,7 @@ list_done_cb (GtkPrintBackend *backend,
   
   g_signal_handlers_disconnect_by_func (backend, list_added_cb, printer_list);
   g_signal_handlers_disconnect_by_func (backend, list_done_cb, printer_list);
+  g_signal_handlers_disconnect_by_func (backend, backend_status_changed, printer_list);
   
   gtk_print_backend_destroy (backend);
   g_object_unref (backend);
@@ -1118,6 +1134,7 @@ list_printers_init (PrinterList     *printer_list,
 		    GtkPrintBackend *backend)
 {
   GList *list, *node;
+  GtkPrintBackendStatus status;
 
   list = gtk_print_backend_get_printer_list (backend);
 
@@ -1132,7 +1149,10 @@ list_printers_init (PrinterList     *printer_list,
 
   g_list_free (list);
 
-  if (gtk_print_backend_printer_list_is_done (backend))
+  g_object_get (backend, "status", &status, NULL);
+  
+  if (status == GTK_PRINT_BACKEND_STATUS_UNAVAILABLE || 
+      gtk_print_backend_printer_list_is_done (backend))
     {
       printer_list->backends = g_list_remove (printer_list->backends, backend);
       gtk_print_backend_destroy (backend);
@@ -1146,6 +1166,9 @@ list_printers_init (PrinterList     *printer_list,
       g_signal_connect (backend, "printer-list-done", 
 			(GCallback) list_done_cb, 
 			printer_list);
+      g_signal_connect (backend, "notify::status", 
+                        (GCallback) backend_status_changed,
+                        printer_list);     
     }
 
   return FALSE;
